package com.casic.missiles.modular.system.dao;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.casic.missiles.modular.system.dto.*;
import com.casic.missiles.modular.system.model.BusWellInfo;
import com.casic.missiles.core.datascope.DataScope;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 闸井信息表 Mapper 接口
 * </p>
 *
 * @author stylefeng123
 * @since 2019-05-13
 */
public interface DataAnalysisMapper extends BaseMapper<BusWellInfo> {

    List<BusWellInfoDto> getAlarmWellList(@Param("scope") DataScope dataScope, @Param("keywords") String keywords, @Param("wellType") String deviceType, @Param("deptid") String deptid);

    List<BusWellInfoDto> getNotAlarmWellList(@Param("scope") DataScope dataScope, @Param("keywords") String keywords, @Param("wellType") String deviceType, @Param("deptid") String deptid);

    DeviceData wellcoverData(@Param("devcode") String devcode);

    DeviceData liquidData(@Param("devcode") String devcode);

    DeviceData harmfulgasData(@Param("devcode") String devcode);

    DeviceData gasData(@Param("devcode") String devcode);

    DeviceData temphumData(@Param("devcode") String devcode);

    DeviceData digData(@Param("devcode") String devcode);

    DeviceData noiseData(@Param("devcode") String devcode);

    DeviceData wellcoverlocaData(@Param("devcode") String devcode);

    DeviceData lgGasData(@Param("devcode") String devcode);//燃气液位

    DeviceData lgLiquidData(@Param("devcode") String devcode);//燃气液位


    BfztCountDataDto bfztCountData(@Param("deptIds") List<Long> deptIds, @Param("isApp") Boolean isApp);
}
